%pointer
%e 100
%p 200
%n 40
%k 50
%a 100
%o 500


%{
/*
 * Copyright 2006, Thomas Maier-Komor
 * LICENSE: GPLv2, see file COPYING for details
 */

#include "log.h"
#include "tokens.h"
#include <assert.h>
#include <string.h>

#ifdef DEBUGLEX
#define debuglex debug
#else
#define debuglex
#endif

char *Buffer;
static size_t bufsize, psz, fill;

%}

WHITESP		[ \t]+
EQUOTE		\\[^\n]
DQUOTED		\"([^"\n\\]|{EQUOTE})+\"
SQUOTED		"'"([^'\n\\]|{EQUOTE})+"'"
ESPACE		"\\ "
UQUOTED		[^ \t\n'"<>]([^ \t\n]|{ESPACE})*
RDIN		"<"
RDOUT		">"
RDOE		">&"

%%

{WHITESP}	{
	debuglex("W ");
	}

{UQUOTED}	{
	char *at = yytext, *esc;
	fill = yyleng;
	Buffer = yytext;
	while (esc = strchr(at,'\\')) {
		at = esc + 1;
		if (*at == ' ') {
			size_t l = fill - (at - Buffer) + 1;
			memmove(esc, at, l);
			--fill;
		}
	}
	Buffer[fill] = 0;
	debuglex("UQ(%s) ",Buffer);
	return UQUOTED;
	}

{DQUOTED}	{
	char *at = yytext + 1, *esc;
	fill = yyleng - 2;
	Buffer = at;
	while (esc = strchr(at,'\\')) {
		at = esc + 1;
		if (*at == '"') {
			size_t l = fill - (at - Buffer) + 1;
			memmove(esc, at, l);
			--fill;
		}
	}
	Buffer[fill] = 0;
	debuglex("DQ(%s) ",Buffer);
	return DQUOTED;
	}

{SQUOTED}	{
	char *at = yytext + 1, *esc;
	fill = yyleng - 2;
	Buffer = at;
	while (esc = strchr(at,'\\')) {
		at = esc + 1;
		if (*at == '\'') {
			size_t l = fill - (at - Buffer) + 1;
			memmove(esc, at, l);
			--fill;
		}
	}
	Buffer[fill] = 0;
	debuglex("S(%s) ",Buffer);
	return SQUOTED;
	}

{RDIN}		{
	debuglex("I ");
	return INPUT;
	}

{RDOUT}		{
	debuglex("O ");
	return OUTPUT;
	}

{RDOE}		{
	debuglex("E ");
	return ERRLOG;
	}

\n		{
	debuglex("E\n");
	return EOL;
	}

.		{
	error("invalid input: %s", yytext);
	}

%%

int yywrap(void)
{
	return 1;
}


#ifdef LEXDEBUG
void sprint(char *s)
{
        dbug(s);
}

void allprint(char c)
{
        dbug("%c",c);
}
#endif

